import stamp.core.*;
import MyProjects.MyLib.*;

/**
 * This program is to control the operation of the KLS laser system.
 * <p>
 * Operation of the laser is controlled via RS232 communication protocol
 * between the KLS system and a DTE PC.
 * The KLS system incorporates a Javelin Stamp PIC to manage the
 * RS232 communication and control the operation of the system.
 * The list of commands is defined in a separate package MyProjects.MyLib.
 *
 * @version 1.0 19 Aug 2005
 * @OFSI - Frederic Durville
 */

public class KLScomDebugTest {
  // Variable definitions
  // Library defining the commands and
  // method matching received character array to pre-defined command.
  static CommandLibrary MyCommandsLib;
  // Pins for RS232 serial Tx and Rx communication
  // and for LCD communication
  final static int SERIAL_TX_PIN = CPU.pin12;
  final static int SERIAL_RX_PIN = CPU.pin13;
  final static int LCDpin = CPU.pin10;

  // Pins for laser control
  final static int ITLK = CPU.pin10;
  final static int SHUTTER = CPU.pin2;
  final static int LSRDY = CPU.pin0;
  final static int LSRON = CPU.pin3;
  final static int LDFLT = CPU.pin1;
  final static int OUTPUT = CPU.pin4;

  final static int CLK = CPU.pin8;     // Pin number for ADC0834 A/D clock signal
  final static int CS = CPU.pin6;      // Pin number for ADC0834 A/D Channel Select signal
  final static int DI = CPU.pin7;      // Pin number for ADC0834 A/D DI signal
  final static int DATA = CPU.pin9;    // Pin number for ADC0834 A/D data signal
  final static int maxPower=80;       // Maximum operating power of 8 Watt
  final static int clkOn = 2;         // Duration of Clock pulse in units of 8.68 microsec

  final static char [] debugRx={'R','c','v','d',':'};    // added for debug to LCD display
  final static char [] debugTx={'S','e','n','t',':'};    // added for debug to LCD display
  final static char [] debugPwr={'p','w','r',':'};    // added for debug to LCD display

  // Other variables
  static char c;
  static int sec=0;
  static int bufferLength=0;
  static int i=0;
  static int ii=0;
  static int com=0;
  static int power=0;
  static int setPower=0;
  static int status=0;
  static int readMonPower=0;
  static int readOutPower=0;
  static int current=0;
  static int duty=0;
  static int slope=73;
  static int threshold=81;
  static int maxCurrent=150;
  static int dataDigital=0;
  static int timeCount=0;

  static boolean interlock=true;
  static int [] calPower= new int[2];


 // Buffers and character arrays used to receive and transmit commnands
 // through the RS232 serial com link
  static char [] transferBuffer = new char [24];
  static char [] conversionBuffer = new char [9];
  static StringBuffer conversionString=new StringBuffer(3);


    // Virtual Peripheral used for serial communication
    static Uart txUart = new Uart( Uart.dirTransmit, SERIAL_TX_PIN, Uart.dontInvert,
                          Uart.speed9600, Uart.stop1 );
    static Uart rxUart = new Uart( Uart.dirReceive, SERIAL_RX_PIN, Uart.dontInvert,
                          Uart.speed9600, Uart.stop1 );
    static Uart LCDcom = new Uart( Uart.dirTransmit, LCDpin, Uart.invert,
                          Uart.speed9600, Uart.stop1 );

    // Virtual Peripheral for setting laser output power
    static DAC outputPower = new DAC(OUTPUT);

    // Virtual Peripheral Timer
    static Timer clock = new Timer();


  // Reading content of the receiving VP buffer only if its length is >7
  // Added checking format validity of received command  29 Aug 05
  // Check that last character is '*' else send error message
  // If received more than 9 characters shift characters to first 9
  static int ReadRx(){
    bufferLength=rxUart.bufferCount();
      if (bufferLength>8){
        if (bufferLength>9) CPU.delay(100);    // Wait 10msec to make sure all trsnsmission is completed
        rxUart.receiveString(transferBuffer,bufferLength,0);  //Uart buffer is automatically cleared when read
        if (transferBuffer[8]!='*'){
          if (transferBuffer[bufferLength-1]!='*')
                      SendTx(MyCommandsLib.StatusChar(7));
          else {
            for (i=0;i<9;i++){
            transferBuffer[i]=transferBuffer[i+bufferLength-9];
            }
          }
        }
      com = MyCommandsLib.MatchCommand(transferBuffer);
        if (com!=0) {
          SendTx(transferBuffer);

    LCDcom.sendByte(254);
    LCDcom.sendByte(131);
    for (i=0;i<transferBuffer.length;i++){
    LCDcom.sendByte(transferBuffer[i]);
    }
    CPU.delay(100);

          if (com==4 && status>1) SetPower(transferBuffer);
          if (com==9) SendStatus(status, setPower);
          for (i=0;i<transferBuffer.length;i++){
            transferBuffer[i]=' ';
          };
        }
      }
    return com;
  }


  // Transmitting content of the TransferBuffer character array
  static void SendTx(char [] transferBuffer){
    txUart.sendString(transferBuffer,transferBuffer.length,0);
    CPU.delay(50);

    LCDcom.sendByte(254);
    LCDcom.sendByte(195);
    for (i=0;i<transferBuffer.length;i++){
    LCDcom.sendByte(transferBuffer[i]);
    }

    CPU.delay(100);

  }


  // Checking and calibrating laser
  // Checked - 30 Aug 05
  static int CheckLaser(){
    i=0;
    readMonPower=0;
    CPU.writePin(LSRDY, false);
    CPU.writePin(LSRON, true);
    for (ii=0;ii<2;ii++){
    current=100+ii*60;
    duty=255*(current/2)/250;
    outputPower.update(duty);
    status=1;

      while (i<20){
        com=ReadRx();
          if (com==1){
            status=0;
            break;
          }
        CPU.delay(1000);
        i++;
        if (i==20) status=2;
      }

    calPower[ii]=ReadPower();
    i=0;
    }
    CPU.writePin(LSRON, false);
    CPU.writePin(LSRDY, true);
    slope=(calPower[1]-calPower[0])*10/6;   // Calculation of slope in Watt/100A
    threshold=(calPower[1]*100-calPower[0]*160)/(calPower[1]-calPower[0]);
    return status;
  }

  // Delay before Ready Mode
  // Completed and verified
  static int Delay(){
      for (ii=0;ii<6;ii++){ // Total delay of 60x50=3000msec
        SendTx(MyCommandsLib.StatusChar(4));
      for (i=0;i<11;i++){
        status=5;
        com=ReadRx();
        if (com==1){      // Received EMGCY command
        status=0;
        com=0;
        break;
        }
        if (com==8){      // Received STDBY command
        status=3;
        com=0;
        break;
        }
        CPU.delay(500);  // Delay of 50 msec - 500 periods of 100 microsec
      }
      }
  return status;
  }


  // Turning laser off and going to Standby mode
  // Completed AND checked  31 Aug 05
  static void LaserStdby(){
  CPU.writePin(LSRDY, true);
  CPU.writePin(LSRON, false);
  CPU.writePin(SHUTTER, false);
  status=3;
  }

  // Emergency shutdown
  // Completed AND checked 31 Aug 05
  static void Emergency(){
  CPU.writePin(LSRON, false);
  CPU.writePin(LSRDY, true);
  CPU.writePin(SHUTTER, false);
  outputPower.update(0);
  status=0;
  com=0;
    LCDcom.sendByte(254);
    LCDcom.sendByte(1);
    CPU.delay(10000);

    LCDcom.sendByte(254);
    LCDcom.sendByte(127);
    LCDcom.sendByte('R');
    LCDcom.sendByte('x');
    LCDcom.sendByte(':');

    CPU.delay(2000);

    LCDcom.sendByte(254);
    LCDcom.sendByte(191);
    LCDcom.sendByte('T');
    LCDcom.sendByte('x');
    LCDcom.sendByte(':');

    CPU.delay(2000);
  }

  // Setting Power Level
  // Completed AND checked 29 Aug 05
  static void SetPower(char [] data){
  setPower=0  ;
  maxCurrent=(maxPower/slope)*100+threshold;
  conversionString.clear();
  for (i=0;i<3;i++) conversionString.append(transferBuffer[i+5]);
  setPower= Integer.parseInt(conversionString);
    if (setPower>80) {
    setPower=0;
    SendTx(MyCommandsLib.StatusChar(8));
    }
  current=(setPower*100)/slope+threshold;
    if (current>250){
    setPower=0;
    current=0;
    SendTx(MyCommandsLib.StatusChar(9));
    }
  duty=255*(current/2)/250;
  outputPower.update(duty);
  }

// Translated from BS2 program
// Completed - tested 30 Aug 05
  static int ReadPower(){
    CPU.writePin(CS,false);               //'Chip Select
    CPU.writePin(DI,true);
// Selecting channel with MUX address on DI (CH0=100 - CH1=110 - CH2=101 - CH3=111)
// selecting CH2
    CPU.pulseOut (clkOn,CLK);
    CPU.writePin(DI,true);                // 'MUX address SGL/DIF
    CPU.pulseOut (clkOn,CLK);
    CPU.writePin(DI,false);               //'MUX address ODD/SIGN
    CPU.pulseOut (clkOn,CLK);
    CPU.writePin(DI,true);                //'MUX address SELECT
//'===End of MUX address
    power=(ReadDataADC()*100)/255;                 //'Value in Watts assuming 2W/V of signal
  return power;
  }
//'===========End of reading data===========


//  Translated from BS2 program
//  Completed AND tested  31 Aug 2005
  static int ReadDataADC(){
    CPU.pulseOut (clkOn,CLK);        //'MUX settling
//'===Start of Data Out reading (MSB first)
    dataDigital=0;
      for (i=1;i<9;i++){
        dataDigital=dataDigital*2;
        CPU.pulseOut (clkOn,CLK);
        if (CPU.readPin(DATA))dataDigital=dataDigital+1;
      }
//'===End of data reading
    CPU.writePin(CS,true);              //          'End of Chip Select
  return dataDigital;
  }


  // Check Interlock - return true if open
  // Completed AND tested  31 Aug 05
  static boolean CheckInterlock(){
    interlock=CPU.readPin(ITLK);
  return interlock;
  }


// Sending Power Level
// Includes converting the power number into a 3-character StringBuffer,
// append it to the $PWR: command CharArray,
// and sending the resulting CharArray.
  static void SendPower(){
    readMonPower=ReadPower();
    conversionString=new StringBuffer(Integer.toString(readMonPower));
    if (conversionString.length()<3)
                conversionString.insert(0,'0');
    if (conversionString.length()<3)
                conversionString.insert(0,'0');
    for (i=0;i<3;i++)
                transferBuffer[i+5]=conversionString.charAt(i);
    transferBuffer[0]='$';
    SendTx(transferBuffer);
  }


// The following method is completed - 31 Aug 05
// The exact format of the information to be sent is yet to be defined - 29 Aug 05
  static void SendStatus(int status, int setPower){
    conversionString=new StringBuffer(Integer.toString(setPower));
    if (conversionString.length()<3)
                conversionString.insert(0,'0');
    if (conversionString.length()<3)
                conversionString.insert(0,'0');
    transferBuffer=MyCommandsLib.StatusChar(status);
    for (i=1;i<4;i++){
        transferBuffer[i]=transferBuffer[i+4];
    }
    for (i=0;i<3;i++){
        transferBuffer[i+5]=conversionString.charAt(i);
    }
    transferBuffer[0]='$';
    SendTx(transferBuffer);
    com=0;
  }

  // Main method.....
  public static void main() {
    Emergency();
    com=0;
    CPU.writePin(LSRON,false);        // Initialize laser controls
    CPU.writePin(LSRDY,true);
    CPU.writePin(SHUTTER,false);


    LCDcom.sendByte(254);
    LCDcom.sendByte(1);
    CPU.delay(10000);

    LCDcom.sendByte(254);
    LCDcom.sendByte(127);
    LCDcom.sendByte('R');
    LCDcom.sendByte('x');
    LCDcom.sendByte(':');

    CPU.delay(2000);

    LCDcom.sendByte(254);
    LCDcom.sendByte(191);
    LCDcom.sendByte('T');
    LCDcom.sendByte('x');
    LCDcom.sendByte(':');

    CPU.delay(2000);

    clock.mark();
    poweron:
    while (true){
    com=ReadRx();
    if (com==1) Emergency();
    if (clock.timeout(1000) && status<6){          // Send status every second
      SendTx(MyCommandsLib.StatusChar(status));
      clock.mark();
    }

    switch (status){

      case 0:
        if (com!=0) SendTx(transferBuffer);
        if (com==4 && status>1) SetPower(transferBuffer);
        if (com==9) SendStatus(status, setPower);

      if (com==3){      // Receive DTERDY command
        clock.mark();
        status=1;
        SendTx(MyCommandsLib.StatusChar(status));
        status=CheckLaser();   // Checking and calibrating laser
        if (status==2) SendTx(MyCommandsLib.StatusChar(status));
        com=0;
      };
      break;

      case 2:               // Laser OK
        if (com==4){     // Received Set Power !PWR: command
          clock.mark();
          status=3;
          SendTx(MyCommandsLib.StatusChar(status));
        };
      break;

      case 3:           //laserstandby:
          if (com==5){          // Received LSRDY command
            status=4;           // going to ready mode
            SendTx(MyCommandsLib.StatusChar(status));
            status=Delay();
            clock.mark();            // activation delay FDA requirement
            if (status==5 && !CheckInterlock() ){
              CPU.writePin(SHUTTER, true);           // Open safety Shutter
              CPU.writePin(LSRDY, false);
              SendTx(MyCommandsLib.StatusChar(status));
            }
          };
      break;

      case 5:              //laserready:
            if (CheckInterlock() || com==8){ // If Interlock OPEN or STDBY command received
              LaserStdby();                  // go to standby mode
              SendTx(MyCommandsLib.StatusChar(status));
              break;
            }

            if (com==6 && status==5){  // Receive LSRON command
              status=6;
              timeCount=0;
              clock.mark();
              com=0;
              SendTx(MyCommandsLib.StatusChar(status));
            };
            break;

      case 6:              //laseron:
      if (com==1)Emergency();  // Received EMGCY command


      if (CheckInterlock() || com==8){ // If Interlock OPEN or STDBY command received
         LaserStdby();                  // go to standby mode
         break;
      }

      if (com==2){                  // Received LSROF command
         CPU.writePin(LSRON, false);
         status=5;                   // go to Ready mode
         break;
      }

      if (com==7){
        SendPower();
        com=0;
      }

      if (com==6){
       com=0;
       timeCount=0;
       clock.mark();
      }
      if (clock.timeout(1000)){
        timeCount++;
        clock.mark();
        SendTx(MyCommandsLib.StatusChar(status));
      }

      if (timeCount==3){        // Timeout for LSRON - 3 sec
        timeCount=0;
        LaserStdby();
        break;
      }

      CPU.writePin(LSRON, true);
      break;

      }
    }
  }
}